<?php
/**
 * @package Wapple_Architect
 * @subpackage WAPL Theme
 */

/**
 * Get WAPL header
 * @param string $type
 * @param array $options
 * @return string
 */
function get_wapl_header($type = 'site', $headerOptions = array())
{
	$file = WP_PLUGIN_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'theme'.DIRECTORY_SEPARATOR.'header.php';
	return include_once($file);
}

/**
 * Get WAPL footer
 * @return string
 */
function get_wapl_footer()
{
	$file = WP_PLUGIN_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'theme'.DIRECTORY_SEPARATOR.'footer.php';
	return include_once($file);
}
/**
 * Get comments code
 * @return string 
 */
function get_wapl_comments()
{
	$file = WP_PLUGIN_DIR.DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'theme'.DIRECTORY_SEPARATOR.'comments.php';
	return include_once($file);
}

/**
 * Debug function
 * @param $debug mixed
 * @access public
 * @return void
 */
if(!function_exists('debug'))
{
	function debug($debug)
	{
		echo '<pre>'.print_r($debug,true).'</pre>';
	}
}

/**
 * Process WAPL into meaningful markup
 * @param $waplString string
 * @access public
 * @return void
 */
function process_wapl($waplString)
{
	if(ARCHITECT_DEBUG)
	{
		header('Content-type: application/xml');
		echo $waplString;
		die();
	}
	
	if(ARCHITECT_DO_SOAP)
	{
		global $waplHeaders;
		global $waplSoapClient;
		
		$params = array(
			'devKey' => get_option('architect_devkey'), 
			'wapl' => $waplString, 
			'deviceHeaders' => $waplHeaders
		);
		
		// Send markup to API and parse through simplexml
		$result = @$waplSoapClient->getMarkupFromWapl($params);
		
		if(is_soap_fault($result))
		{
			if(strpos($result->faultstring, 'Account API request limit reached') === 0)
			{
				setcookie('architectError', $result->faultstring, time()+1800, '/');
			}
			setcookie('isMobile', "0", time()+1800, '/');
			header('Location:'.get_permalink());
			exit();
		} else
		{
			setcookie('architectError', "", time()-3600, '/');
		}
		
		$xml = simplexml_load_string($result);
		
		foreach($xml->header->item as $val)
		{
			header($val);
		}
		
		// Flush output buffer - to clean up any other plugin mess!
		ob_end_clean();
		
		// Echo correct markup
		echo trim($xml->markup);
		die();
	} else if(ARCHITECT_DO_REST)
	{
		global $waplHeaders;
		
		$postfields = array(
			'devKey' => get_option('architect_devkey'),
			'wapl' => $waplString,
			'headers' => $waplHeaders
		);
		
		$c = curl_init();
		curl_setopt($c, CURLOPT_URL, 'http://webservices.wapple.net/getMarkupFromWapl.php');
		curl_setopt($c, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($c, CURLOPT_POST, 1);
		curl_setopt($c, CURLOPT_POSTFIELDS, $postfields);
		
		$result = curl_exec($c);
		if(strpos($result, 'Account API request limit reached')!== false)
		{
			setcookie('architectError', 'Account API request limit reached', time()+1800, '/');
			setcookie('isMobile', "0", time()+1800, '/');
			header('Location:'.get_permalink());
			exit();
		} else if(strpos($result, 'Developer key authentication error') !== false)
		{
			setcookie('architectError', 'Developer key authentication error', time()+1800, '/');
			setcookie('isMobile', "0", time()+1800, '/');
			header('Location:'.get_permalink());
			exit();
		} else
		{
			setcookie('architectError', "", time()-3600, '/');
		}
		curl_close($c);
		
		$xml = simplexml_load_string($result);
		foreach($xml->header->item as $val)
		{
			header($val);
		}
		
		// Flush output buffer - to clean up any other plugin mess!
		ob_end_clean();
		
		// Echo correct markup
		echo trim($xml->markup);
		die();
		
	} else
	{
		header('Content-type: application/xml');
		echo $waplString;
	}
}

require_once('wapl_builder.php');
/**
 * WordPress WAPL parser
 * 
 * Extends the waplBuilder class to build perfect markup
 * @author Rich Gubby
 */
class WordPressWapl extends waplBuilder
{
	/**
	 * Foreign characters and their HTML ASCII equivalent. Quicker to define rather than use the "ord" function.
	 * @access public
	 * @var array
	 */
	var $chars = array(
		// French
		'&uuml;' => '&#220;',
		'&Agrave;' => '&#192;', '&agrave;' => '&#224;', '&Acirc;' => '&#194;', '&acirc;' => '&#226;', '&AElig;' => '&#198;', '&aelig;' => '&#230;',
		'&Ccedil;' => '&#199;', '&ccedil;' => '&#231;', '&Egrave;' => '&#200;', '&egrave;' => '&#232;', '&Eacute;' => '&#201;', '&eacute;' => '&#233;',
	 	'&Ecirc;' => '&#202;', '&ecirc;' => '&#234;', '&Euml;' => '&#203;', '&euml;' => '&#235;', '&Icirc;' => '&#206;', '&icirc;' => '&#238;',
		'&Iuml;' => '&#207;', '&iuml;' => '&#239;', '&Ocirc;' => '&#212;', '&ocirc;' => '&#244;', '&OElig;' => '&#140;', '&oelig;' => '&#156;',
		'&Ugrave;' => '&#217;', '&ugrave;' => '&#249;', '&Ucirc;' => '&#219;', '&ucirc;' => '&#251;', '&Uuml;' => '&#220;', '&uuml;' => '&#252;',
		'&euro;' => '&#8364;',
		// German
		'&Auml;' => '&#195;', '&auml;' => '&#228;', '&Ouml;' => '&#214;', '&ouml;' => '&#246;', '&szlig;' => '&#223;',
		// Some random codes we need to clean up
		'&tilde;' => '&#126;', '&sbquo;' => '&#130;', '&dbquo;' => '&#132;', '&dagger;' => '&#134;', '&Dagger;' => '&#135;', '&lsaquo;' => '&#139;',
		'&lsquo;' => '&#145;', '&rsquo;' => '&#146;', '&ldquo;' => '&#147;', '&rdquo;' => '&#148;', '&ndash;' => '&#150;', '&mdash;' => '&#151;',
		'&trade;' => '&#153;', '&rsaquo;' => '&#155;', 'iexcl;' => '&#161;', '&brvbar;' => '&#166;', '&copy;' => '&#169;', '&ordf;' => '&#170;',
		'&not;' => '&#172;', '&shy;' => '&#173;', '&reg' => '&#174;', '&deg;' => '&#176;', '&sup2;' => '&#178;', '&sup3;' => '&#179;',
		'&micro;' => '&#181;', '&para;' => '&#182;', '&middot' => '&#183;', '&sup1;' => '&#185;', '&ordm;' => '&#186;', '&iquest;' => '&#191;',
		'&sect;' => '&#167;', '&oline;' => '&#8254;'
	);
	
	var $charsOriginal = array(
	
		// Cyrillic
		'А' => '&#1040;','а' => '&#1072;','Б' => '&#1041;','б' => '&#1073;','В' => '&#1042;','в' => '&#1074;',
		'Г' => '&#1043;','г' => '&#1075;','Д' => '&#1044;','д' => '&#1076;','Е' => '&#1045;','е' => '&#1077;',
		'Ж' => '&#1046;','ж' => '&#1078;','З' => '&#1047;','з' => '&#1079;','И' => '&#1048;','и' => '&#1080;',
		'Й' => '&#1049;','й' => '&#1081;','К' => '&#1050;','к' => '&#1082;','Л' => '&#1051;','л' => '&#1083;',
		'М' => '&#1052;','м' => '&#1084;','Н' => '&#1053;','н' => '&#1085;','О' => '&#1054;','о' => '&#1086;',
		'П' => '&#1055;','п' => '&#1087;','Р' => '&#1056;','р' => '&#1088;','С' => '&#1057;','с' => '&#1089;',
		'Т' => '&#1058;','т' => '&#1090;','У' => '&#1059;','у' => '&#1091;','Ф' => '&#1060;','ф' => '&#1092;',
		'Х' => '&#1061;','х' => '&#1093;','Ц' => '&#1062;','ц' => '&#1094;','Ч' => '&#1063;','ч' => '&#1095;',
		'Ш' => '&#1064;','ш' => '&#1096;','Щ' => '&#1065;','щ' => '&#1097;','Ъ' => '&#1066;','ъ' => '&#1098;',
		'Ы' => '&#1067;','ы' => '&#1099;','Ь' => '&#1068;','ь' => '&#1100;','Э' => '&#1069;','э' => '&#1101;',
		'Ю' => '&#1070;','ю' => '&#1102;','Я' => '&#1071;','я' => '&#1103;',
		// French
		'À' => '&#192;','à' => '&#224;','Â' => '&#194;','â' => '&#226;','Æ' => '&#198;',
		'æ' => '&#230;','Ç' => '&#199;','ç' => '&#231;','È' => '&#200;','è' => '&#232;',
		'É' => '&#201;','é' => '&#233;','Ê' => '&#202;','ê' => '&#234;','Ë' => '&#203;',
		'ë' => '&#235;','Î' => '&#206;','î' => '&#238;','Ï' => '&#207;','ï' => '&#239;',
		'Ô' => '&#212;','ô' => '&#244;','Œ' => '&#140;','œ' => '&#156;','Ù' => '&#217;',
		'ù' => '&#249;','Û' => '&#219;','û' => '&#251;','Ü' => '&#220;','ü' => '&#252;',
		'€' => '&#8364;','₣' => '&#8355;', '&uuml;' => '&#220;',
		// German
		'Ä' => '&#196;','ä' => '&#228;','Ö' => '&#214;','ö' => '&#246;','ß' => '&#223;',
		'°' => '&#176;',
	);
	
	/**
	 * Format content into WAPL readable format
	 * @param string $content
	 * @param integer $imagescale
	 * @param integer $imagequality
	 * @param string $class
	 * @param integer $length
	 * @param string $transcol
	 * @access public
	 * @return string
	 */
	function format_text($content, $imagescale = 95, $imagequality = 90, $class = 'entry', $length = null, $transcol = '')
	{
		// Remove comments
		$content = str_replace(
			array('&#8211;', '<strong><strong>', '&copy;', '&nbsp;'), 
			array('--', '<strong>', '&#169;', '&#160;'), 
			$content
		);

		$content = $this->foreignChars($content);
		
		$replacements = array();
		require_once('simple_html_dom.php');
		
		$html = str_get_html($content);
		
		// Remove comments
		foreach($html->find('comment') as $element)
		{
			$content = str_ireplace($element->outertext, '', $content);
		}
		
		// Remove script tags
		foreach($html->find('script') as $element)
		{
			$replacements[trim($element->outertext())] = '';
		}
		
		// Any special filters for other plugins
		$content = $this->__filters($content, $class);
		
		// Get rid of the "more" links - don't need these for mobile, they clutter things up
		if(!get_option('architect_home_showmore'))
		{
			foreach($html->find('a.more-link') as $element)
			{
				$replacements[trim($element->outertext())] = '';
			}
		}
		
		// Replace [caption] with <caption>
		$content = preg_replace('/(\[caption(.*?)\])(.*?)(\[\/caption\])/i', '<caption ${2}>${3}</caption>', $content);
		
		foreach($replacements as $key => $val)
		{
			$content = str_ireplace(trim($key), $val, $content);
		}
		
		// Use parent format text
		$content = parent::format_text($content, $imagescale, $imagequality, $class, $length, $transcol);
		
		return utf8_encode(str_replace('& ', '&amp; ', $content));
	}
	
	/**
	 * Check if a string is UTF8 or not
	 * 
	 * @param string $string
	 * @access public
	 * @return string
	 */
	function is_utf8($string) 
	{
	    return (preg_match('/^([\x00-\x7f]|[\xc0-\xdf][\x80-\xbf]|[\xe0-\xef][\x80-\xbf]{2}|[\xf0-\xf7][\x80-\xbf]{3}|[\xf8-\xfb][\x80-\xbf]{4}|[\xfc-\xfd][\x80-\xbf]{5})*$/', $string) === 1);
	}
	
	/**
	 * Convert foreign characters to ascii alternatives
	 * 
	 * @param string $string
	 * @access public
	 * @return string
	 */
	function foreignChars($string)
	{
		// Add support for Japanese, Chinese, Korean, etc characters
		if(function_exists('mb_detect_encoding'))
		{
			$newString = '';
			for ($i=0; $i < mb_strlen($string, 'UTF-8'); $i++)
			{
		        $ch = mb_substr($string, $i, 1, 'UTF-8');
		        
		        if($ch && trim($ch) != '')
		        {
		        	$returnVal = $this->uniord($ch);
		        	if($returnVal['encode'] == true)
		        	{
		        		$newString .= '&#'.$returnVal['ud'].';';
		        	} else
		        	{
		        		$newString .= mb_substr($string, $i, 1, 'UTF-8');
		        	}
		        } else
		        {
		        	$newString .= mb_substr($string, $i, 1, 'UTF-8');
		        }
		    }
		    
		    $string = $newString;
		    
		    // Convert any other characters
		    $string = str_replace(array_keys($this->chars), array_values($this->chars), $string);
		} else
		{
			// Czech, Slovak and Slovenian
			// http://webdesign.about.com/od/localization/l/blhtmlcodes-cz.htm
			
			// Greek
			//http://webdesign.about.com/od/localization/l/blhtmlcodes-gr.htm
			
			// Hawaiian
			//http://webdesign.about.com/od/localization/l/blhtmlcodes-hi.htm
			
			// Italian
			//http://webdesign.about.com/od/localization/l/blhtmlcodes-it.htm
			
			// Polish
			//http://webdesign.about.com/od/localization/l/blhtmlcodes-pl.htm
			
			// Romanian
			//http://webdesign.about.com/od/localization/l/blhtmlcodes-ro.htm
			
			// Spanish
			//http://webdesign.about.com/od/localization/l/blhtmlcodes-sp.htm
			
			// Turkish
			//http://webdesign.about.com/od/localization/l/blhtmlcodes-tr.htm
			// Convert characters manually
			$string = str_replace(
				array_keys(array_merge($this->chars, $this->charsOriginal)), 
				array_values(array_merge($this->chars, $this->charsOriginal)), 
				$string
			);
		}
		
		// Any other characters
		$string = architectCharsOther($string);

		return utf8_decode($string);
	}
	
	function uniord($c)
	{
		$ud = 0;
		$encode = true;
		if (ord($c{0})>=0 && ord($c{0})<=127)
		{
			$encode = false;
			$ud = $c{0};
		}
		if (ord($c{0})>=192 && ord($c{0})<=223)
		{
			$ud = (ord($c{0})-192)*64 + (ord($c{1})-128);
		}
		if (ord($c{0})>=224 && ord($c{0})<=239)
		{
			$ud = (ord($c{0})-224)*4096 + (ord($c{1})-128)*64 + (ord($c{2})-128);
		}
		if (ord($c{0})>=240 && ord($c{0})<=247)
		{
			$ud = (ord($c{0})-240)*262144 + (ord($c{1})-128)*4096 + (ord($c{2})-128)*64 + (ord($c{3})-128);
		}
		if (ord($c{0})>=248 && ord($c{0})<=251)
		{
			$ud = (ord($c{0})-248)*16777216 + (ord($c{1})-128)*262144 + (ord($c{2})-128)*4096 + (ord($c{3})-128)*64 + (ord($c{4})-128);
		}
		if (ord($c{0})>=252 && ord($c{0})<=253)
		{
			$ud = (ord($c{0})-252)*1073741824 + (ord($c{1})-128)*16777216 + (ord($c{2})-128)*262144 + (ord($c{3})-128)*4096 + (ord($c{4})-128)*64 + (ord($c{5})-128);
		}
		if (ord($c{0})>=254 && ord($c{0})<=255)
		{
			//error
			$ud = false;
		}
		return array('ud' => $ud, 'encode' => $encode);
	}

	/**
	 * Special filters written specifically for other plugins
	 * @param string $content
	 * @param string $class
	 * @access public
	 * @return string
	 */
	function __filters($content, $class)
	{
		// Path to extensions
		$path = '';
		if(defined('WP_PLUGIN_DIR'))
		{
			$path = WP_PLUGIN_DIR;
		} else if(defined('ABSPATH'))
		{
			$path = ABSPATH.'wp-content'.DIRECTORY_SEPARATOR.'plugins';
		}
		$path .= DIRECTORY_SEPARATOR.get_wapl_plugin_base().DIRECTORY_SEPARATOR.'extensions';
		
		if ($handle = opendir($path)) 
		{
			// Get all files in the extensions directory
			while (false !== ($file = readdir($handle))) 
			{
				if ($file != '.' && $file != '..' && $file != '.svn') 
				{
					// Include the file
					include_once($path.DIRECTORY_SEPARATOR.$file);
					
					// Get class name - should match the filename
					$className = substr($file,0,strpos($file,'.'));
					
					// Create a new class
					if(class_exists($className))
					{
						$filterClass = new $className;
					
						if(method_exists($filterClass, 'format'))
						{
							// Return formatted text
							$content = $filterClass->format($content, $class);
						}
					} 
				}
			}
			closedir($handle);
		}
		return $content;
	}
}

/**
 * Load a WAPL element
 * @param $element string
 * @access public
 * @return void
 */
function get_wapl_element($element, $queryStringOptions = array())
{
	require_once('simple_html_dom.php');
	return require('elements/'.$element.'.php');
}

function get_wapl_response_string($type, $count, $title = '')
{
	if(!isset($type) OR $type == '')
	{
		$type = 1;
	}
	$text = array();
	
	$text[1] = array('format' => "%d %s", 'single' => 'response', 'plural' => 'responses');
	$text[2] = array('format' => "%d %s %s", 'single' => 'response to', 'plural' => 'responses to');
	$text[3] = array('format' => "%d %s", 'single' => 'comment', 'plural' => 'comments');
	$text[4] = array('format' => "%d %s %s", 'single' => 'comment to', 'plural' => 'comments to');

	$singletext = get_option('architect_single_commentsingletext');
	$pluraltext = get_option('architect_single_commentpluraltext');
	if($singletext && $pluraltext)
	{
		if($count == 0 || $count > 1)
		{
			return sprintf($text[$type]['format'], $count, stripslashes($pluraltext), $title);
		} else
		{
			return sprintf($text[$type]['format'], $count, stripslashes($singletext), $title);
		}
	} else
	{
		if($count == 0 || $count > 1)
		{
			return sprintf($text[$type]['format'], $count, $text[$type]['plural'], $title);
		} else
		{
			return sprintf($text[$type]['format'], $count, $text[$type]['single'], $title);
		}
	}
}

if(!function_exists('get_author_posts_url'))
{
	function get_author_posts_url($author_id, $author_nicename = '') {
		global $wp_rewrite;
		$auth_ID = (int) $author_id;
		$link = $wp_rewrite->get_author_permastruct();
	
		if ( empty($link) ) {
			$file = get_option('home') . '/';
			$link = $file . '?author=' . $auth_ID;
		} else {
			if ( '' == $author_nicename ) {
				$user = get_userdata($author_id);
				if ( !empty($user->user_nicename) )
					$author_nicename = $user->user_nicename;
			}
			$link = str_replace('%author%', $author_nicename, $link);
			$link = get_option('home') . trailingslashit($link);
		}
	
		$link = apply_filters('author_link', $link, $author_id, $author_nicename);
	
		return $link;
	}
}
?>